export declare type EventCallback = (val: any) => void;
export declare enum MarkupMode {
    markdown = 0,
    text = 1,
}
export interface MarkupOptions {
    content?: string;
    custom?: any;
    dirtyLimit?: number;
    fontName?: string;
    fontSize?: number;
    followLinks?: boolean;
    highlight?: string;
    idleDelay?: number;
    mode?: MarkupMode;
    onChange?: EventCallback;
    onClick?: EventCallback;
    onClickLink?: EventCallback;
}
export declare class Markup {
    private _dirtyIdle;
    private _dirtyInline;
    private _dirtyCount;
    private _dirtyLimit;
    private _delay;
    private _changing;
    private _idle;
    private _idleDelay;
    private _idleTimer;
    private _editor;
    private _fonts;
    private _line;
    private _modes;
    private _opts;
    private _paste;
    private _processor;
    private _quill;
    constructor(quill: any, opts?: MarkupOptions);
    readonly editor: HTMLElement;
    readonly editorKey: any;
    readonly fonts: string[];
    readonly highlights: string[];
    readonly modes: any[];
    readonly opts: MarkupOptions;
    readonly quill: any;
    /**
     * Resets the idle timer when the user does something in the app.
     */
    private resetInactivityTimer();
    /**
     * When the user is idle for N seconds this function is called by a global
     * timer to set a flag and rescan the full document.  This can be an
     * expensive operation and we need to find the tradeoff limit
     */
    private markIdle();
    /**
     * Calls the quill history redo function
     */
    redo(): void;
    /**
     * Rescans the entire document for highlighting.  This is a wrapper around
     * the processor's full refresh function.
     */
    refresh(): void;
    /**
     * Changes the current highlighting mode and display style.  It also sets
     * the content within control.
     * @param opts {HighlightOptions} configuration options.  `mode` sets the
     * editing mode.  `styling` sets the current display theme.  `custom` is
     * an object that contains custom format colors (for highlights).  The
     * `content` resets the content within the control.  If this is null it
     * is ignored.
     */
    set(opts: MarkupOptions): void;
    /**
     * Will call the current processor's bold function to make the current
     * highlight or word bold.
     */
    setBold(): void;
    /**
     * Will update the current content of the control with the given
     * @param content {string} the new content settting for the editor.
     */
    setContent(content: string): void;
    /**
     * Changes the current overall font for the editor.  This control works with
     * mono fonts, so this will set it for all text.  The fonts are all defined
     * in `./lib/fonts`.
     * @param fontName {string} the name of the font to set.
     */
    setFont(fontName: string): void;
    /**
     * Changes the current overall size of the fonts within the editor.
     * @param fontSize {number} the number of pixels in the font sizing.  This
     * will resolve to `##px`.
     */
    setFontSize(fontSize: number): void;
    /**
     * Calls the processor's current header creation function
     * @param level {string} the level selected by the user
     */
    setHeader(level: string): void;
    setHighlight(name: string): void;
    /**
     * Calls the current processor's italic function to make the current
     * highlight or word italic.
     */
    setItalic(): void;
    /**
     * Changes the current display mode for the markup processor
     * @param mode {string} the name of the mode to set
     */
    setMode(mode: string): void;
    /**
     * Calls the current processor's strikthrogh function to highlight the
     * current word/selection.
     */
    setStrikeThrough(): void;
    /**
     * Calls the current processor's underline function to highlight the
     * current word/selection.
     */
    setUnderline(): void;
    /**
     * Calls the quill history undo function
     */
    undo(): void;
    /**
     * This event is invoked whenever the mouse is clicked within the editor.
     * It will call the user give onClick handler and pass the position within
     * the editor that was clicked.
     *
     * If the `followLinks` configuration option is given, then another handler
     * is called that will check if the clicked position is a link.  If it is,
     * then the regex match object that found this link is returned to the
     * callback (giving the full text, start/end offsets, and groups).
     */
    private handleClick();
    /**
     * This event is invoked whenever a change occurs in the editor (any type).
     * @param eventName {string} the name of the event that occurred.
     * @param args {any[]} the dyanamic parameter list passed to this event.
     */
    private handleEditorChange(eventName, ...args);
    /**
     * When the user pastes information into the editor, this event is fired
     */
    private handlePaste();
    /**
     * Invoked with each change of the content text.
     *
     * This will also hold a timer to peform a full scan after N seconds.
     * Using "sections" to format has a trade off where a block format on
     * a boundary of a section can lose formatting where the bottom of the
     * block is seen by the section, but the top is not, so the regex string
     * will not match within the section.  This timer will send a full
     * range of the document to the processor after N seconds (5 seconds by
     * default).  The timer will only occur when changes occur (so it is
     * idle if the keyboard is idle)
     */
    private handleTextChange(delta?, old?, source?);
}
