import { BinaryTree, SortedList } from 'util.ds';
import { Match } from 'util.matches';
import { Section } from 'util.section';
export interface MatchData {
    key: string;
    link?: Match;
}
export declare abstract class BaseMarkupMode {
    private static readonly INLINE_SIZE;
    protected _blocksDirty: boolean;
    protected _blocks: SortedList<number>;
    protected _blockID: BinaryTree<string>;
    protected _delta: any;
    protected _end: number;
    protected _links: BinaryTree<MatchData>;
    protected _pos: number;
    protected _quill: any;
    protected _range: any;
    protected _start: number;
    protected _style: any;
    protected _subText: string;
    protected _text: string;
    protected _admonition: RegExp;
    protected _code: RegExp;
    protected _formulaInline: RegExp;
    protected _formulaBlock: RegExp;
    protected _url: RegExp;
    protected _wiki: RegExp;
    constructor(quill: any);
    readonly end: number;
    readonly areBlocksDirty: boolean;
    /**
     * @return {Section} the line at the current cursor.  It is given as a
     * Section block.
     */
    readonly line: Section;
    readonly links: BinaryTree<MatchData>;
    /**
     * @return the current position within the buffer
     */
    pos: number;
    /**
     * @return a reference to the Quill instance for this editor instance
     */
    readonly quill: any;
    /**
     * @return {any} the current range object of the user selection/cursor
     */
    range: any;
    /**
     * @return {Section} the word at the current cursor or the user highlighted
     * region.  It is contained within a Section block.
     */
    readonly selection: Section;
    readonly start: number;
    style: any;
    /**
     * @return {string} when the highlight function is called it works with a
     * substring.  This function returns the last substring computed when
     * a change is detected.
     */
    readonly subText: string;
    /**
     * @return {string} the current text string of the editor buffer.
     */
    /**
     * Replaces the current text content in Quill with the given start
     * @param val {string} the string used to replace current content in the editor
     */
    text: string;
    abstract handleBold(): void;
    abstract handleHeader(level: number): void;
    abstract handleItalic(): void;
    abstract handleStrikeThrough(): void;
    abstract handleUnderline(): void;
    highlightBlock(): void;
    highlightInline(): void;
    /**
     * Takes a selection area from a document and applies a markup annotation
     * to the beginning and end of the selection area.  This only works on a
     * single line (inline)
     * @param selection {Section} the text/location where the annotation will
     * be applied
     * @param chevron {string} the character string that will surround the
     * selection
     */
    annotateInline(selection: Section, chevron: string): any;
    /**
     * Takes a selection area from the document and applies a surrounding block
     * markup annotation (such as header).  This version allows the caller to
     * set a start/end chevron value to surround the block.
     * @param selection {Section} the text/location where the annotation will
     * be applied.
     * @param startChevron {string} the string that will be placed on the front
     * of the block.
     * @param [endChevron] {string} the string that will be placed on the end
     * of the block.
     */
    annotateBlock(selection: Section, startChevron: string, endChevron?: string): any;
    /**
     * Searches for a fenced code region and applies syntax highlighting to it.
     * @param text {string} the text buffer where the regex will look for
     * matches.
     * @param re {RegExp} the regular expression used for the search
     * @return {Delta} the Delta created by this change
     */
    codify(text: string, re: RegExp): any;
    /**
     * Applies a single color format to the given buffer based on the given regex
     * string. This uses the selected section from the main buffer to compute the
     * start offset.  This doesn't search the whole buffer, but a "subText"
     * region of the buffer given to the function.
     * @param text {string} the text buffer where the regex will look for
     * matches.
     * @param re {RegExp} the regular expression used for the search
     * @param color {string} the color string (hex or named) used for the
     * formatting of the color.
     * @return {Delta} the Delta created by this change
     */
    colorize(text: string, re: RegExp, color: string): any;
    /**
     * An inline color processor that uses regex groups to color a 3 part token
     * The regex for this should have three groups:
     *
     * 1. initial chevron (like parens, bracket)
     * 2. token string
     * 3. closing chevron
     *
     * The color for 1 & 3 are determined by the chevron style color.  The token
     * colors are set using the styling parameter object passed to the function.  This
     * object should have `color` and/or `background` set to change the colors for The
     * matched string.
     * @param re {RegExp} the regular expression used for the search
     * @param color {string} the color string (hex or named) used for the
     * formatting of the color.
     * @return {Delta} the Delta created by this change
     */
    colorizeGroup(text: string, re: RegExp, styling: any): any;
    /**
     * Applies a single color over a given buffer based on the given regex.
     * This call can span multiple lines in the regex, where the regular
     * colorize works on a single line.
     * @param text {string} the text buffer where the regex will look for
     * matches.
     * @param re {RegExp} the regular expression used for the search
     * @param color {string} the color string (hex or named) used for the
     * formatting of the color.
     * @return {Delta} the Delta created by this change
     */
    colorizeBlock(text: string, re: RegExp, color: string): any;
    /**
     * Special colorization function that adds colors to embedded links.  The
     * given regex values must break the the string into three matching parts:
     *
     * - match 1 - the link name
     * - match 2 - the url link
     * - match 3 - an optional title value
     *
     * The color values for the link constituent parts is located in
     * highlights.json.  They are: linkName, link, linkTitle, and linkChevron
     *
     * @param text {string} the text buffer where the regex will look for
     * matches.
     * @param re {RegExp} the regular expression used for the search
     * @return {Delta} the delta structure created
     */
    colorizeLink(text: string, re: RegExp, styling?: any): any;
    /**
     * When the document changes this function is invoked to reapply the
     * highlighting.  This is the base class function that saves the section
     * range and removes the formatting from that range.  The mode is then
     * responsible for reapplying the format to that range via callback.
     * @param start {number} the starting location within the full document
     * where markdown changes should be scanned.
     * @param end {number} the ending location within the full document
     * where markdown changes should be scanned
     */
    handleChange(start: number, end: number): void;
    /**
     * Scans the known block elements within the document and reapplies the formatting.
     */
    refreshBlock(): void;
    /**
     * Scans the entire document to reapply hihglighting.  All formatting rules are
     * removed and reapplied.  This is expensive and slow, so use with caution on
     * large documents.  This function is exposed to the markup interface so that the
     * user could create a refresh button.  It is usd with an initial document that
     * is placed in the control, or when a paste occurs.
     */
    refreshFull(): void;
    /**
     * Scans a section of the document and applies formatting to the inline sections
     * only.  It will skip block sections and only attempt to process lines that
     * are not part of a block.
     */
    refreshInline(): void;
    /**
     * Resets all block detail elements back to their starting point.
     */
    private clearBlockDetails();
    /**
     * Retrieves an array of blocks that would appear within an insline
     * section of the document.  It searches for points where the Section
     * may bisect the block element and fixes the start/end index of the
     * block when that happens.  The returned blocks should always have
     * an even number of members (each block must have a start/end)
     * @param start {number} the starting position within the section where
     * the block may be.
     * @param end {number} the ending position within the section where
     * the block may be
     * @return {number[]} an array containing the start/end locations for
     * blocks that reside within a region.
     */
    private getBlocks(start, end);
    private processBlockTokens(tokens, styling);
    private processCodeTokens(tokens);
    private processInlineTokens(tokens, styling);
    private processInlineGroupTokens(tokens, styling);
    private processLinkTokens(tokens, styling);
    /**
     * Takes a string and a given regex, finds matches, and then loops through
     * each match calling a handler function to process the matches.  The
     * handler function is dynamic.  There are two start locations: INLINE and
     * BLOCK.  The INLINE type computes a start location based on the current
     * line position.  The BLOCK type uses the location of the first match
     * in the regex within the full document.
     * @param text {string} the data the regex will use to find matches.
     * @param re {RegExp} the regular expression used to find matching tokens
     * @param handler {Function} the function that will process all of the
     * matches found by the regex matcher.
     * @param styling {any} custom styling object used by the handler.  This
     * is just passed through to the handler function.
     * @return {Delta} the delta object generate from the regex matches.
     */
    private processRegex(text, re, fn, parseType, styling?);
    /**
     * When a block structure is encountered its details like the start/end offset
     * of the block are saved.  It also creates a hash of the block area text.  These
     * details are used to determine when updates should occur to block sections.
     *
     * When `refreshBlocks` is called, the details of this function are used to
     * determine if any of the current blocks were new (with `blocksDirty`).
     *
     * The `blocksDirty' is set to true when a block's hash is not found.
     *
     * When that is true, a new block was encountered, so refresh must occur for
     * block elements.  When it is false, none of the blocks changed, so they do not
     * need to have their formats removed and reparsed for highlights (which is more
     * expensive and visually uglier than this code).
     *
     * @param match [Match] the section of the document that matached a regex for
     * a block element.
     */
    private saveBlockDetails(match);
}
